%{
%}
%token LOOP_START LOOP_END
%token PREV NEXT
%token PLUS MINUS
%token READ WRITE

%type <node> Instruction InstructionList Loop 

%{
#include <stdio.h>
#include "il_system.h"
#include "il_opcodes.h"
#include "il_meta.h"
#include "il_utils.h"

#include "bf_rename.h"
#include "bf_internal.h"

int yydebug;

/*
 * Global code generator object.
 */
ILGenInfo BFCodeGen;

/*
 * Imports from the lexical analyser.
 */

extern int yylex();


#ifdef YYTEXT_POINTER
extern char *bf_text;
#else
extern char bf_text[];
#endif

static void yyerror(char *msg)
{
	CCPluginParseError(msg, bf_text);
}
%}

%union	{
	ILNode	*node;
};

%start CompilationUnit

%%

CompilationUnit
	: /* empty */ {
		CCError("input file is empty");
	}
	|
	InstructionList {
		CCParseTree = ILNode_BFProgram_create($1);
	} 
	;

InstructionList
	:Instruction {
		$$ = ILNode_List_create();
		ILNode_List_Add($$,$1);
	}
	| InstructionList Instruction {
		$$ = $1;
		ILNode_List_Add($$,$2);
	}

Loop
	: LOOP_START InstructionList LOOP_END {
		$$ = ILNode_BFLoop_create($2);
	}

Instruction
	: READ{	
		$$ = ILNode_BFRead_create();
	}
	| WRITE{
		$$ = ILNode_BFWrite_create();
	}
	| PLUS {
		$$ = ILNode_BFPlus_create();
	}
	| MINUS{
		$$ = ILNode_BFMinus_create();
	}
	| PREV {
		$$ = ILNode_BFPrev_create();
	}
	| NEXT {
		$$ = ILNode_BFNext_create();
	}
	| Loop {
		$$ = $1;
	}
%%
